@@ -117,7 +117,10 @@ class BaseAdapter(object):
     def close(self, conn):
         conn.close()
 
-    def lookup_cast(self, lookup, value):
+    def op_override(self, field, op, value):
+        return op
+
+    def lookup_cast(self, field, lookup, value):
         """
         When a lookup is being performed as a part of a WHERE clause, provides
         a way to alter the incoming value that is passed to the database driver
@@ -161,7 +164,7 @@ class SqliteAdapter(BaseAdapter):
             raise ImproperlyConfigured('sqlite3 must be installed on the system')
         return sqlite3.connect(database, **kwargs)
 
-    def lookup_cast(self, lookup, value):
+    def lookup_cast(self, field, lookup, value):
         if lookup == 'contains':
             return '*%s*' % value
         elif lookup == 'icontains':
@@ -326,6 +329,9 @@ class Database(object):
         logger.debug((sql, params))
         return cursor
 
+    def begin(self):
+        pass
+
     def commit(self):
         self.get_conn().commit()
 
@@ -344,6 +350,7 @@ class Database(object):
         def inner(*args, **kwargs):
             orig = self.get_autocommit()
             self.set_autocommit(False)
+            self.begin()
             try:
                 res = func(*args, **kwargs)
                 self.commit()
@@ -378,30 +385,32 @@ class Database(object):
             )
 
     def field_sql(self, field):
-        return '%s %s' % (self.quote_name(field.db_column), field.render_field_template())
+        rendered = field.render_field_template(self.adapter.quote_char)
+        return '%s %s' % (self.quote_name(field.db_column), rendered)
 
-    def create_table_query(self, model_class, safe, extra=''):
+    def get_column_sql(self, model_class):
+        return map(self.field_sql, model_class._meta.get_fields())
+
+    def create_table_query(self, model_class, safe, extra='', framing=None):
         if model_class._meta.pk_sequence and self.adapter.sequence_support:
             if not self.sequence_exists(model_class._meta.pk_sequence):
                 self.create_sequence(model_class._meta.pk_sequence)
-        framing = safe and "CREATE TABLE IF NOT EXISTS %s (%s)%s;" or "CREATE TABLE %s (%s)%s;"
-        columns = []
-
-        for field in model_class._meta.get_fields():
-            columns.append(self.field_sql(field))
+        framing = framing or 'CREATE TABLE %s%s (%s)%s;'
+        safe_str = safe and 'IF NOT EXISTS ' or ''
+        columns = self.get_column_sql(model_class)
 
         if extra:
             extra = ' ' + extra
 
         table = self.quote_name(model_class._meta.db_table)
 
-        return framing % (table, ', '.join(columns), extra)
+        return framing % (safe_str, table, ', '.join(columns), extra)
 
     def create_table(self, model_class, safe=False, extra=''):
         self.execute(self.create_table_query(model_class, safe, extra))
 
-    def create_index_query(self, model_class, field_name, unique):
-        framing = 'CREATE %(unique)s INDEX %(index)s ON %(table)s(%(field)s);'
+    def create_index_query(self, model_class, field_name, unique, framing=None):
+        framing = framing or 'CREATE %(unique)s INDEX %(index)s ON %(table)s(%(field)s);'
 
         if field_name not in model_class._meta.fields:
             raise AttributeError(
@@ -567,7 +576,7 @@ class MySQLDatabase(Database):
             self.quote_name(model_class._meta.db_table),
             self.quote_name(field.db_column),
             self.quote_name(new_name),
-            field.render_field_template(),
+            field.render_field_template(self.adapter.quote_char),
         )
 
     def get_indexes_for_table(self, table):
@@ -587,6 +596,7 @@ class transaction(object):
     def __enter__(self):
         self._orig = self.db.get_autocommit()
         self.db.set_autocommit(False)
+        self.db.begin()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         if exc_type:
@@ -891,7 +901,7 @@ class R(Leaf):
         self.params = params
         super(R, self).__init__()
 
-    def sql_select(self):
+    def sql_select(self, model_class):
         if len(self.params) == 2:
             return self.params
         else:
@@ -900,6 +910,9 @@ class R(Leaf):
     def sql_where(self):
         return self.params[0], self.params[1:]
 
+    def sql_update(self):
+        return self.params[0], self.params[1]
+
 
 def apply_model(model, item):
     """
@@ -990,8 +1003,8 @@ class BaseQuery(object):
     def qn(self, name):
         return self.database.quote_name(name)
 
-    def lookup_cast(self, lookup, value):
-        return self.database.adapter.lookup_cast(lookup, value)
+    def lookup_cast(self, field, lookup, value):
+        return self.database.adapter.lookup_cast(field, lookup, value)
 
     def parse_query_args(self, _model, **query):
         """
@@ -1018,6 +1031,8 @@ class BaseQuery(object):
                 if field is None:
                     raise
 
+            op = self.database.adapter.op_override(field, op, rhs)
+
             if isinstance(rhs, R):
                 expr, params = rhs.sql_where()
                 lookup_value = [field.db_value(o) for o in params]
@@ -1046,18 +1061,18 @@ class BaseQuery(object):
                 elif op == 'isnull':
                     operation = 'IS NULL' if rhs else 'IS NOT NULL'
                     lookup_value = []
-                elif isinstance(rhs, (list, tuple)):
-                    # currently this only happens on 'between' lookups, but leave
-                    # it general to lists and tuples
+                elif op == 'between':
                     lookup_value = [field.db_value(o) for o in rhs]
-                    operation = self.operations[op] % \
-                        tuple(self.interpolation for v in lookup_value)
+                    operation = self.operations[op] % (self.interpolation, self.interpolation)
+                elif isinstance(rhs, (list, tuple)):
+                    lookup_value = [field.db_value(o) for o in rhs]
+                    operation = self.operations[op] % self.interpolation
                 else:
                     lookup_value = field.db_value(rhs)
                     operation = self.operations[op] % self.interpolation
 
             parsed.append(
-                (field.db_column, (operation, self.lookup_cast(op, lookup_value)))
+                (field.db_column, (operation, self.lookup_cast(field, op, lookup_value)))
             )
 
         return parsed
@@ -1379,7 +1394,7 @@ class SelectQuery(BaseQuery):
         else:
             clone.query = 'COUNT(%s)' % (clone.model._meta.pk_col)
 
-        res = clone.database.execute(*clone.sql())
+        res = clone.database.execute(*clone.sql(), require_commit=False)
 
         return (res.fetchone() or [0])[0]
 
@@ -1390,7 +1405,7 @@ class SelectQuery(BaseQuery):
         sql, params = clone.sql()
         query = 'SELECT COUNT(1) FROM (%s) AS wrapped_select' % sql
 
-        res = clone.database.execute(query, params)
+        res = clone.database.execute(query, params, require_commit=False)
 
         return res.fetchone()[0]
 
@@ -1453,7 +1468,7 @@ class SelectQuery(BaseQuery):
     def exists(self):
         clone = self.paginate(1, 1)
         clone.query = '(1) AS a'
-        curs = self.database.execute(*clone.sql())
+        curs = self.database.execute(*clone.sql(), require_commit=False)
         return bool(curs.fetchone())
 
     def get(self, *args, **kwargs):
@@ -1479,7 +1494,7 @@ class SelectQuery(BaseQuery):
     def aggregate(self, func):
         clone = self.order_by()
         clone.query = [func]
-        curs = self.database.execute(*clone.sql())
+        curs = self.database.execute(*clone.sql(), require_commit=False)
         return curs.fetchone()[0]
 
     @returns_clone
@@ -1500,7 +1515,7 @@ class SelectQuery(BaseQuery):
             elif q in (self.model._meta.pk_col, self.model._meta.pk_name):
                 q = {self.model: [self.model._meta.pk_name]}
             else:
-                return q, [], False
+                return q, [], [], False
 
         # by now we should have a dictionary if a valid type was passed in
         if not isinstance(q, dict):
@@ -1512,6 +1527,7 @@ class SelectQuery(BaseQuery):
         # normalize if we are working with a dictionary
         columns = []
         model_cols = []
+        sparams = []
 
         for model, alias in sorted_models:
             if model not in q:
@@ -1524,12 +1540,21 @@ class SelectQuery(BaseQuery):
                 q[model] =  q[model][:idx] + model._meta.get_field_names() + q[model][idx+1:]
 
             for clause in q[model]:
-                if isinstance(clause, R):
-                    clause = clause.sql_select()
+                if hasattr(clause, 'sql_select'):
+                    clause = clause.sql_select(model)
 
                 if isinstance(clause, tuple):
                     local_columns = False
-                    if len(clause) == 3:
+                    if len(clause) > 3:
+                        template, col_name, col_alias = clause[:3]
+                        cparams = clause[3:]
+                        column = model._meta.get_column(col_name)
+                        columns.append(template % \
+                            (self.safe_combine(model, alias, column), col_alias)
+                        )
+                        sparams.extend(cparams)
+                        model_cols.append((model, (template, column, col_alias)))
+                    elif len(clause) == 3:
                         func, col_name, col_alias = clause
                         column = model._meta.get_column(col_name)
                         columns.append('%s(%s) AS %s' % \
@@ -1544,13 +1569,13 @@ class SelectQuery(BaseQuery):
                         )
                         model_cols.append((model, (column, col_alias)))
                     else:
-                        raise ValueError('Clause must be either a 2- or 3-tuple')
+                        raise ValueError('Unknown type in select query')
                 else:
                     column = model._meta.get_column(clause)
                     columns.append(self.safe_combine(model, alias, column))
                     model_cols.append((model, column))
 
-        return ', '.join(columns), model_cols, (models_queried == 1 and local_columns)
+        return ', '.join(columns), model_cols, sparams, (models_queried == 1 and local_columns)
 
     def sql_meta(self):
         joins, clauses, alias_map = self.compile_where()
@@ -1574,7 +1599,8 @@ class SelectQuery(BaseQuery):
             for field in clause:
                 group_by.append(self.safe_combine(model, alias, field))
 
-        parsed_query, model_cols, simple = self.parse_select_query(alias_map)
+        parsed_query, model_cols, sparams, simple = self.parse_select_query(alias_map)
+        params.extend(sparams)
         query_meta = {
             'columns': model_cols,
             'graph': self._joins,
@@ -1677,7 +1703,7 @@ class UpdateQuery(BaseQuery):
                 if field is None:
                     raise
 
-            if not isinstance(v, F):
+            if not isinstance(v, (F, R)):
                 v = field.db_value(v)
 
             sets[field.db_column] = v
@@ -1697,6 +1723,10 @@ class UpdateQuery(BaseQuery):
         for k, v in sorted(set_statement.items(), key=lambda (k, v): k):
             if isinstance(v, F):
                 value = self.parse_f(v, v.model or self.model, alias_map)
+            elif isinstance(v, R):
+                value, rparams = v.sql_update()
+                value = value % self.interpolation
+                params.append(rparams)
             else:
                 params.append(v)
                 value = self.interpolation
@@ -2201,10 +2231,11 @@ class Field(object):
     def get_column(self):
         return self.column_class(**self.attributes)
 
-    def render_field_template(self):
+    def render_field_template(self, quote_char=''):
         params = {
             'column': self.column.render(self.model._meta.database),
             'nullable': ternary(self.null, '', ' NOT NULL'),
+            'qc': quote_char,
         }
         params.update(self.column.attributes)
         return self.field_template % params
@@ -2356,7 +2387,7 @@ class ReverseForeignRelatedObject(object):
 
 
 class ForeignKeyField(IntegerField):
-    field_template = '%(column)s%(nullable)s REFERENCES %(to_table)s (%(to_pk)s)%(cascade)s%(extra)s'
+    field_template = '%(column)s%(nullable)s REFERENCES %(qc)s%(to_table)s%(qc)s (%(qc)s%(to_pk)s%(qc)s)%(cascade)s%(extra)s'
 
     def __init__(self, to, null=False, related_name=None, cascade=False, extra=None, *args, **kwargs):
         self.to = to
